"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const constants_1 = require("./constants");
const utils_1 = require("./utils");
const methods_1 = require("./methods");
const adapters_1 = require("./adapters");
/**
 * Extended Buffer Class
 *
 * @export
 * @class BuffExt
 */
class BuffExt {
    /**
     * Creates an instance of BuffExt.
     * @param {Buffer} buffer
     * @param {Endian} [endian='LE'] little or big endian
     * @memberof BuffExt
     */
    constructor(buffer, endian = 'LE') {
        this.buffer = buffer;
        this.endian = endian;
    }
    /**
     * Creates an instance of BuffExt and
     * allocates a new buffer of {size} octets.
     *
     * @static
     * @param {number} size
     * @param {Endian} [endian]
     * @returns
     * @memberof BuffExt
     */
    static alloc(size, fill, endian) {
        return new BuffExt(Buffer.alloc(size, fill), endian);
    }
    /**
     * When passed a reference to the .buffer property of a TypedArray instance,
     * the newly created Buffer will share the same allocated memory as the TypedArray.
     * The optional {byteOffset} and {length} arguments specify a memory range
     * within the {arrayBuffer} that will be shared by the Buffer.
     *
     * @param arrayBuffer The .buffer property of any TypedArray or a new ArrayBuffer()
     */
    static fromArrayBuffer(arrayBuffer, byteOffset, length, endian) {
        return new BuffExt(Buffer.from(arrayBuffer, byteOffset, length), endian);
    }
    ;
    /**
     * Creates a new Buffer using the passed {data}
     * @param data data to create a new Buffer
     */
    static fromNumberArray(data, endian) {
        return new BuffExt(Buffer.from(data), endian);
    }
    ;
    static fromUint8Array(data, endian) {
        return new BuffExt(Buffer.from(data), endian);
    }
    ;
    /**
     * Creates a new Buffer containing the given JavaScript string {str}.
     * If provided, the {encoding} parameter identifies the character encoding.
     * If not provided, {encoding} defaults to 'utf8'.
     */
    static fromString(str, encoding, endian) {
        return new BuffExt(Buffer.from(str, encoding));
    }
    ;
    /**
     * Returns a new BuffExt class with
     * a completely new buffer that has
     * no reference to the original
     * buffer.
     *
     * @returns {BuffExt}
     * @memberof BuffExt
     */
    clone() {
        const clonedBuffer = utils_1.CloneBuffer(this.buffer);
        return new BuffExt(clonedBuffer, this.endian);
    }
    swap64() {
        return this.buffer.swap64();
    }
    swap16() {
        return this.buffer.swap16();
    }
    swap32() {
        return this.buffer.swap32();
    }
    /**
     * Swaps the high and low WORDS (UINT16)
     *
     * @example [1, 2, 3, 4] => [2, 1, 4, 3]
     *
     * @memberof BuffExt
     */
    swapWords() {
        return this.buffer = utils_1.WordSwap(this.buffer);
    }
    /**
     * The length of bytes of the array
     *
     * @readonly
     * @memberof BuffExt
     */
    get byteLength() {
        return this.buffer.byteLength;
    }
    /**
     * Get the bit at pos.bit
     *
     * @param {number} pos
     * @param {BitIndex} bit
     * @returns {boolean}
     * @memberof BuffExt
     */
    getBitAt(pos, bit) {
        return methods_1.GetBitAt(this.buffer, pos, bit, this.endian);
    }
    /**
     * Set the bit at pos.bit
     *
     * @param {number} pos
     * @param {BitIndex} bit
     * @param {boolean} value
     * @returns {void}
     * @memberof BuffExt
     */
    setBitAt(pos, bit, value) {
        return methods_1.SetBitAt(this.buffer, pos, bit, value, this.endian);
    }
    /**
     * Toggles the bit at pos.bit
     *
     * @param {number} pos
     * @param {BitIndex} bit
     * @returns {void}
     * @memberof BuffExt
     */
    toggleBitAt(pos, bit) {
        return methods_1.ToggleBitAt(this.buffer, pos, bit);
    }
    /**
     * Gets a nibble at a specific byte in the buffer
     *
     * @param {number} pos
     * @param {NibbleSignificance} significance
     * @returns {number}
     * @memberof BuffExt
     */
    getNibbleAt(pos, significance) {
        return methods_1.GetNibbleAt(this.buffer, pos, significance, this.endian);
    }
    /**
     * Sets a nibble at a specific byte in the buffer
     *
     * @param {number} pos
     * @param {NibbleSignificance} significance
     * @param {number} nibbleValue
     * @memberof BuffExt
     */
    setNibbleAt(pos, significance, nibbleValue) {
        methods_1.SetNibbleAt(this.buffer, pos, significance, nibbleValue, this.endian);
    }
    /**
     * Get 8 bit signed value -128..127
     *
     * @param {number} pos
     * @returns
     * @memberof BuffExt
     */
    getSIntAt(pos) {
        return methods_1.GetInt8At(this.buffer, pos);
    }
    /**
     * Set 8 bit signed value -128..127
     *
     * @param {number} pos
     * @param {number} value
     * @returns
     * @memberof BuffExt
     */
    setInt8At(pos, value) {
        return methods_1.SetInt8At(this.buffer, pos, value);
    }
    /**
     * Get 16 bit signed value -32768..32767
     *
     * @param {number} pos
     * @returns
     * @memberof BuffExt
     */
    getInt16At(pos) {
        return methods_1.GetInt16At(this.buffer, pos, this.endian);
    }
    /**
     * Set 16 bit signed value -32768..32767
     *
     * @param {number} pos
     * @param {number} value
     * @returns
     * @memberof BuffExt
     */
    setInt16At(pos, value) {
        return methods_1.SetInt16At(this.buffer, pos, value, this.endian);
    }
    /**
     * Get 32 bit signed value -2147483648..2147483647
     *
     * @param {number} pos
     * @returns
     * @memberof BuffExt
     */
    getInt32At(pos) {
        return methods_1.GetInt32At(this.buffer, pos, this.endian);
    }
    /**
     * Set 32 bit signed value -2147483648..2147483647
     *
     * @param {number} pos
     * @param {number} value
     * @returns
     * @memberof BuffExt
     */
    setInt32At(pos, value) {
        return methods_1.SetInt32At(this.buffer, pos, value, this.endian);
    }
    /**
     *
     *
     * @param {number} pos
     * @returns
     * @memberof BuffExt
     */
    getInt64_UNSAFE(pos) {
        return methods_1.GetInt64_UNSAFE(this.buffer, pos, this.endian);
    }
    /**
     *
     *
     * @param {number} pos
     * @param {number} value
     * @returns
     * @memberof BuffExt
     */
    setInt64_UNSAFE(pos, value) {
        return methods_1.SetInt64_UNSAFE(this.buffer, pos, value, this.endian);
    }
    /**
     * Get 8 bit unsigned value 0..255
     *
     * @param {number} pos
     * @returns
     * @memberof BuffExt
     */
    getUInt8At(pos) {
        return methods_1.GetUInt8At(this.buffer, pos, this.endian);
    }
    /**
     * Gets a binary string representation of the target byte
     *
     * @param {number} pos
     * @param {(BitOrder)} [order=DEFAULT_BIT_ORDER]
     * @returns
     * @memberof BuffExt
     */
    getByteBinaryStringAt(pos, order = constants_1.DEFAULT_BIT_ORDER) {
        return methods_1.GetByteBinaryString(this.buffer, pos, order);
    }
    /**
     * Set 8 bit unsigned value 0..255
     *
     * @param {number} pos
     * @param {number} value
     * @returns
     * @memberof BuffExt
     */
    setUInt8At(pos, value) {
        return methods_1.SetUInt8At(this.buffer, pos, value, this.endian);
    }
    /**
     * Get 16 bit unsigned value 0..65535
     *
     * @param {number} pos
     * @returns
     * @memberof BuffExt
     */
    getUInt16At(pos) {
        return methods_1.GetUInt16At(this.buffer, pos, this.endian);
    }
    /**
     * Gets a binary string representation of the target byte
     *
     * @param {number} pos
     * @param {(BitOrder)} [order=DEFAULT_BIT_ORDER]
     * @returns
     * @memberof BuffExt
     */
    getUInt16BinaryStringAt(pos, order = constants_1.DEFAULT_BIT_ORDER) {
        return methods_1.GetUInt16BinaryString(this.buffer, pos, order);
    }
    /**
     * Set 16 bit unsigned value 0..65535
     *
     * @param {number} pos
     * @param {number} value
     * @returns
     * @memberof BuffExt
     */
    setUInt16At(pos, value) {
        return methods_1.SetUInt16At(this.buffer, pos, value, this.endian);
    }
    /**
     * Get 32 bit unsigned value 0..4294967296
     *
     * @param {number} pos
     * @returns
     * @memberof BuffExt
     */
    getUInt32At(pos) {
        return methods_1.GetUInt32At(this.buffer, pos, this.endian);
    }
    /**
     * Set 32 bit unsigned value 0..4294967295
     *
     * @param {number} pos
     * @param {number} value
     * @returns
     * @memberof BuffExt
     */
    setUInt32At(pos, value) {
        return methods_1.SetUInt32At(this.buffer, pos, value, this.endian);
    }
    /**
     * Get 32 bit float value
     *
     * @param {number} pos
     * @param {number} [precision=3]
     * @returns
     * @memberof BuffExt
     */
    getFloat32At(pos, precision = 3) {
        return methods_1.GetFloat32At(this.buffer, pos, precision, this.endian);
    }
    /**
     * Set 32 bit float value
     *
     * @param {number} pos
     * @param {number} value
     * @returns
     * @memberof BuffExt
     */
    setFloat32At(pos, value) {
        return methods_1.SetFloat32At(this.buffer, pos, value, this.endian);
    }
    /**
     * Get 32 bit float value
     *
     * @param {number} pos
     * @param {number} [precision=3]
     * @returns
     * @memberof BuffExt
     */
    getFloat64At(pos, precision = 3) {
        return methods_1.GetFloat64At(this.buffer, pos, precision, this.endian);
    }
    /**
     * Set 32 bit float value
     *
     * @param {number} pos
     * @param {number} value
     * @returns
     * @memberof BuffExt
     */
    setFloat64At(pos, value) {
        return methods_1.SetFloat64At(this.buffer, pos, value, this.endian);
    }
    /**
     * Get String of specific length from the current buffer
     *
     * @param {number} pos starting position
     * @param {number} length number of characters in the string
     * @param {BufferEncoding} [format=DEFAULT_STRING_ENCODING]
     * @memberof BuffExt
     */
    getStringAt(pos, length, format = constants_1.DEFAULT_STRING_ENCODING) {
        return methods_1.GetStringAt(this.buffer, pos, length, format);
    }
    /**
     * Get Binary-Coded Decimal value at specific byte
     *
     * @param {number} pos
     * @returns {number}
     * @memberof BuffExt
     */
    getBCDAt(pos) {
        return methods_1.GetBCDAt(this.buffer, pos);
    }
    /**
     * Set Binary-Coded Decimal value at specific byte
     *
     * @param {number} pos
     * @returns {number}
     * @memberof BuffExt
     */
    setBCDAt(pos, bcdValue) {
        return methods_1.SetBCDAt(this.buffer, pos, bcdValue);
    }
    /**
     * Get S7 DateTime from buffer at a starting position
     *
     * @param {number} pos starting byte in buffer
     * @returns {Date}
     * @memberof BuffExt
     */
    getS7DateTimeAt(pos) {
        return methods_1.GetS7DateTimeAt(this.buffer, pos);
    }
    /**
     * Set S7 DateTime from buffer at a starting position
     *
     * @param {number} pos starting byte in buffer
     * @param {Date} date date to write into the buffer
     * @returns {void}
     * @memberof BuffExt
     */
    setS7DateTimeAt(pos, date) {
        return methods_1.SetS7DateTimeAt(this.buffer, pos, date);
    }
    /**
     * Gets an IPV4 IP Address from the buffer at a specific position,
     * and returns it as a number array of length 4.
     * @param {number} pos
     * @returns {IPV4Array}
     * @memberof BuffExt
     */
    getIPV4AddressAt(pos) {
        return methods_1.GetIPV4AddressAt(this.buffer, pos);
    }
    /**
     *Gets an IPV4 IP Address from the buffer at a specific position,
     * and converts it into the format: `${ip1}.${ip2}.${ip3}.${ip4}`
     * @param {number} pos
     * @returns {string}
     * @memberof BuffExt
     */
    getIPV4AddressAsStringAt(pos) {
        return methods_1.GetIPV4AddressAsStringAt(this.buffer, pos);
    }
    /**
     * Sets an IPV4 IP Address in the buffer at a specific position
     *
     * @param {number} pos
     * @param {(IPV4Array | number[] | Buffer | ArrayLike<number> | string)} ipv4Address
     * @returns {void}
     * @memberof BuffExt
     */
    setIPV4AddressAt(pos, ipv4Address) {
        return methods_1.SetIPV4AddressAt(this.buffer, pos, ipv4Address);
    }
    /**
     *
     *
     * @returns
     * @memberof BuffExt
     */
    toInt8Array() {
        return new Int8Array(this.buffer);
    }
    /**
     *
     *
     * @returns
     * @memberof BuffExt
     */
    toUint8Array() {
        return new Uint8Array(this.buffer);
    }
    /**
     *
     *
     * @returns
     * @memberof BuffExt
     */
    toUint8ClampedArray() {
        return new Uint8ClampedArray(this.buffer);
    }
    /**
     *
     *
     * @returns
     * @memberof BuffExt
     */
    toInt16Array() {
        return new Int16Array(this.buffer);
    }
    /**
     *
     *
     * @returns
     * @memberof BuffExt
     */
    toUint16Array() {
        return new Uint16Array(this.buffer);
    }
    /**
     *
     *
     * @returns
     * @memberof BuffExt
     */
    toInt32Array() {
        return new Int32Array(this.buffer);
    }
    /**
     *
     *
     * @returns
     * @memberof BuffExt
     */
    toUint32Array() {
        return new Uint32Array(this.buffer);
    }
    /**
     *
     *
     * @returns
     * @memberof BuffExt
     */
    toFloat32Array() {
        return new Float32Array(this.buffer);
    }
    /**
     *
     *
     * @returns
     * @memberof BuffExt
     */
    toFloat64Array() {
        return new Float64Array(this.buffer);
    }
    /**
     * Converts a buffer to a string array
     * where each string is 8 characters of
     * either a 1 or a 0.
     *
     * @param {BitOrder} [order=DEFAULT_BIT_ORDER]
     * @returns
     * @memberof BuffExt
     */
    toBitStringArray(order = constants_1.DEFAULT_BIT_ORDER) {
        return adapters_1.BufferToBitStringArray(this.buffer, this.endian, order);
    }
}
exports.BuffExt = BuffExt;
//# sourceMappingURL=BuffExt.js.map