"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const constants_1 = require("../../constants");
const validators_1 = require("../../validators");
/**
 * Get 64 bit signed value -9223372036854775808..9223372036854775807
 *
 * @export
 * @param {Buffer} buffer
 * @param {number} pos
 * @returns {bigint}
 */
function GetLIntAt(buffer, pos, endian = constants_1.DEFAULT_ENDIAN) {
    validators_1.ValidateBigIntSupported();
    switch (endian) {
        case 'BE':
            return buffer.readBigInt64BE(pos);
        case 'LE':
            return buffer.readBigInt64LE(pos);
    }
}
exports.GetLIntAt = GetLIntAt;
/**
 * Get 64 bit signed value -9223372036854775808..9223372036854775807
 *
 * Node.js versions below 12.0.0 do not support `bigint` data type,
 * so the following code will attempt to calculate the Int64 value,
 * but at the outer ranges these values will have rounding errors.
 *
 * @export
 * @param {Buffer} buffer
 * @param {number} pos
 * @returns {number}
 */
function GetLIntAt_UNSAFE(buffer, pos, endian = constants_1.DEFAULT_ENDIAN) {
    let Result;
    const workingBuffer = Buffer.allocUnsafe(8);
    buffer.copy(workingBuffer, 0, pos, pos + 8);
    switch (endian) {
        case 'BE':
            break;
        case 'LE':
            workingBuffer.swap64();
            break;
    }
    Result = workingBuffer[pos];
    Result <<= 8;
    Result += workingBuffer[pos + 1];
    Result <<= 8;
    Result += workingBuffer[pos + 2];
    Result <<= 8;
    Result += workingBuffer[pos + 3];
    Result <<= 8;
    Result += workingBuffer[pos + 4];
    Result <<= 8;
    Result += workingBuffer[pos + 5];
    Result <<= 8;
    Result += workingBuffer[pos + 6];
    Result <<= 8;
    Result += workingBuffer[pos + 7];
    return Result;
}
exports.GetLIntAt_UNSAFE = GetLIntAt_UNSAFE;
/**
 * Get 64 bit signed value -9223372036854775808..9223372036854775807
 *
 * @export
 * @param {Buffer} buffer
 * @param {number} pos
 * @returns {bigint}
 * @alias GetLIntAt
 */
function GetInt64(buffer, pos, endian = constants_1.DEFAULT_ENDIAN) {
    return GetLIntAt(buffer, pos, endian);
}
exports.GetInt64 = GetInt64;
/**
 * Get 64 bit signed value -9223372036854775808..9223372036854775807
 *
 * Node.js versions below 12.0.0 do not support `bigint` data type,
 * so the following code will attempt to calculate the Int64 value,
 * but at the outer ranges these values will have rounding errors.
 *
 * @export
 * @param {Buffer} buffer
 * @param {number} pos
 * @returns {number}
 * @alias GetLIntAt_UNSAFE
 */
function GetInt64_UNSAFE(buffer, pos, endian = constants_1.DEFAULT_ENDIAN) {
    return GetLIntAt_UNSAFE(buffer, pos, endian);
}
exports.GetInt64_UNSAFE = GetInt64_UNSAFE;
/**
 * Set 64 bit signed value -9223372036854775808..9223372036854775807
 *
 * @export
 * @param {Buffer} buffer
 * @param {number} pos
 * @param {bigint} value
 */
function SetLIntAt(buffer, pos, value, endian = constants_1.DEFAULT_ENDIAN) {
    validators_1.ValidateBigIntSupported();
    switch (endian) {
        case 'BE':
            buffer.writeBigInt64BE(value, pos);
            break;
        case 'LE':
            buffer.writeBigInt64LE(value, pos);
            break;
    }
    return;
}
exports.SetLIntAt = SetLIntAt;
/**
 * Set 64 bit signed value -9223372036854775808..9223372036854775807
 *
 * Node.js versions below 12.0.0 do not support `bigint` data type,
 * so the following code will attempt to calculate the Int64 value,
 * but at the outer ranges these values will have rounding errors.
 *
 * @export
 * @param {Buffer} buffer
 * @param {number} pos
 * @param {number} value
 */
function SetLIntAt_UNSAFE(buffer, pos, value, endian = constants_1.DEFAULT_ENDIAN) {
    const workingBuffer = Buffer.allocUnsafe(8);
    workingBuffer[pos + 7] = (value & 0xFF);
    workingBuffer[pos + 6] = ((value >> 8) & 0xFF);
    workingBuffer[pos + 5] = ((value >> 16) & 0xFF);
    workingBuffer[pos + 4] = ((value >> 24) & 0xFF);
    workingBuffer[pos + 3] = ((value >> 32) & 0xFF);
    workingBuffer[pos + 2] = ((value >> 40) & 0xFF);
    workingBuffer[pos + 1] = ((value >> 48) & 0xFF);
    workingBuffer[pos] = ((value >> 56) & 0xFF);
    switch (endian) {
        case 'BE':
            break;
        case 'LE':
            workingBuffer.swap64();
            break;
    }
    workingBuffer.copy(buffer, 0, pos, pos + 8);
    return;
}
exports.SetLIntAt_UNSAFE = SetLIntAt_UNSAFE;
/**
 * Set 64 bit signed value -9223372036854775808..9223372036854775807
 *
 * @export
 * @param {Buffer} buffer
 * @param {number} pos
 * @param {bigint} value
 * @alias SetLIntAt
 */
function SetInt64(buffer, pos, value, endian = constants_1.DEFAULT_ENDIAN) {
    return SetLIntAt(buffer, pos, value, endian);
}
exports.SetInt64 = SetInt64;
/**
 * Set 64 bit signed value -9223372036854775808..9223372036854775807
 *
 * Node.js versions below 12.0.0 do not support `bigint` data type,
 * so the following code will attempt to calculate the Int64 value,
 * but at the outer ranges these values will have rounding errors.
 *
 * @export
 * @param {Buffer} buffer
 * @param {number} pos
 * @param {number} value
 * @alias SetLIntAt_UNSAFE
 */
function SetInt64_UNSAFE(buffer, pos, value, endian = constants_1.DEFAULT_ENDIAN) {
    return SetLIntAt_UNSAFE(buffer, pos, value, endian);
}
exports.SetInt64_UNSAFE = SetInt64_UNSAFE;
//# sourceMappingURL=int64.js.map